/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 2001-2004
 *	Sleepycat Software.  All rights reserved.
 *
 * $Id: os_id.c,v 12.1 2005/01/19 13:52:59 bostic Exp $
 */

#include "db_config.h"

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>

#include <stdlib.h>
#include <unistd.h>
#endif

#include "db_int.h"

/*
 * __os_id --
 *	Return the current process ID.
 *
 * PUBLIC: void __os_id __P((pid_t *));
 */
void
__os_id(idp)
	pid_t *idp;
{
#ifdef	HAVE_VXWORKS
	*idp = (pid_t)taskIdSelf();
#else
	*idp = getpid();
#endif
}

/*
 * __os_unique_id --
 *	Return a unique 32-bit value.
 *
 * PUBLIC: void __os_unique_id __P((DB_ENV *, u_int32_t *));
 */
void
__os_unique_id(dbenv, idp)
	DB_ENV *dbenv;
	u_int32_t *idp;
{
	static int first = 1;
	pid_t pid;
	u_int32_t id, sec, usec;

	*idp = 0;

	/*
	 * Our randomized value is comprised of our process ID, the current
	 * time of day and a couple of a stack addresses, all XOR'd together.
	 */
	__os_id(&pid);
	__os_clock(dbenv, &sec, &usec);

	id = (u_int32_t)pid ^ sec ^ usec ^ P_TO_UINT32(&pid);

	/*
	 * We could try and find a reasonable random-number generator, but
	 * that's not all that easy to do.  Seed and use srand()/rand(), if
	 * we can find them.
	 */
#if HAVE_SRAND
	if (first == 1)
		srand((u_int)id);
#endif
	first = 0;

#if HAVE_RAND
	id ^= (u_int)rand();
#endif

	*idp = id;
}
